<?php

namespace Drupal\terms_export_import\Form;

use Drupal\Core\File\Exception\FileException;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Entity\File;
use Drupal\file\Validation\FileValidatorInterface;
use Drupal\terms_export_import\Service\TermsExportImportService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class TermsImportForm. Provides the import form for terms.
 */
class TermsImportForm extends FormBase {

    /**
     * Custom CSV import service.
     *
     * @var \Drupal\terms_export_import\Service\TermsExportImportService
     */
    protected TermsExportImportService $exportImportService;

    /**
     * The file system service.
     *
     * @var \Drupal\Core\File\FileSystemInterface
     */
    protected FileSystemInterface $fileSystem;

    /**
     * The file validator services.
     *
     * @var \Drupal\file\Validation\FileValidatorInterface
     */
    protected FileValidatorInterface $fileValidator;

    /**
     * {@inheritdoc}
     */
    public static function create(ContainerInterface $container): TermsImportForm {
        $instance = parent::create($container);
        $instance->exportImportService = $container->get('terms_export_import.service');
        $instance->fileSystem = $container->get('file_system');
        $instance->fileValidator = $container->get('file.validator');
        return $instance;
    }

    /**
     * {@inheritdoc}
     */
    public function getFormId(): string {
        return 'terms_import_form';
    }

    /**
     * {@inheritdoc}
     */
    public function buildForm(array $form, FormStateInterface $form_state): array {
        // A simple file input (not managed_file).
        $form['import_file'] = [
            '#type'        => 'file',
            '#title'       => $this->t('CSV file'),
            '#description' => $this->t('Upload a CSV file with columns: name,description,parent,weight'),
            '#required'    => TRUE,
        ];

        // (Optional) vocabulary autocomplete
        // selector for selecting which vocab to import into.
        $form['vocabulary'] = [
            '#type'        => 'entity_autocomplete',
            '#title'       => $this->t('Vocabulary'),
            '#target_type' => 'taxonomy_vocabulary',
            '#required'    => TRUE,
            '#description' => $this->t('Select the vocabulary to import into.'),
        ];

        $form['actions'] = [
            '#type' => 'actions',
        ];
        $form['actions']['submit'] = [
            '#type'  => 'submit',
            '#value' => $this->t('Import CSV'),
        ];

        return $form;
    }

    /**
     * {@inheritdoc}
     */
    public function validateForm(array &$form, FormStateInterface $form_state): void {
        // 1) Retrieve the raw Symfony UploadedFile from the request.
        $uploaded_files = \Drupal::request()->files->get('files', []);
        /** @var \Symfony\Component\HttpFoundation\File\UploadedFile|null $uploaded_file */
        $uploaded_file = $uploaded_files['import_file'] ?? NULL;

        if (!$uploaded_file || !$uploaded_file->isValid()) {
            $form_state->setErrorByName('import_file', $this->t('Please upload a valid CSV file.'));
            return;
        }

        // 2) Write the file data to
        // temporary:// so we can create a Drupal file entity.
        $filename = $uploaded_file->getClientOriginalName();
        $temp_directory = 'temporary://taxonomy_import_export';
        $this->fileSystem->prepareDirectory(
            $temp_directory,
            FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS
        );
        $temp_uri = $temp_directory . '/' . $filename;

        // Remove existing file with that name (optional).
        $realpath = $this->fileSystem->realpath($temp_uri);
        if ($realpath && file_exists($realpath)) {
            $this->fileSystem->delete($temp_uri);
        }

        // Write the content into temporary://.
        file_put_contents($this->fileSystem->realpath($temp_uri), file_get_contents($uploaded_file->getRealPath()));

        // 3) Create a temporary Drupal file entity for the just-written file.
        $file_entity = File::create([
            'uri'    => $temp_uri,
            // 0 => temporary
            'status' => 0,
        ]);
        $file_entity->setFilename($filename);
        $file_entity->save();

        // 4) Validate the file using the new constraint plugins.
        $validators = [
            'FileExtension' => ['extensions' => 'csv'],
        ];

        // The result is a Symfony ConstraintViolationListInterface if errors occur.
        $violations = $this->fileValidator->validate($file_entity, $validators);

        if ($violations->count() > 0) {
            // If there’s a violation,
            // delete the physical file & entity, set an error.
            $this->fileSystem->delete($temp_uri);
            $file_entity->delete();

            // Show the first violation, or combine them if you prefer.
            /** @var \Symfony\Component\Validator\ConstraintViolationInterface $first_violation */
            $first_violation = $violations[0];
            $message = $this->t('File validation error: @msg', ['@msg' => $first_violation->getMessage()]);

            $form_state->setErrorByName('import_file', $message);
            return;
        }

        // 5) If no violations,
        // store the file ID so we can finalize in submitForm().
        $form_state->setValue('temp_file_id', $file_entity->id());
    }

    /**
     * {@inheritdoc}
     */
    public function submitForm(array &$form, FormStateInterface $form_state): void {
        // Load the file entity we created in validateForm().
        $fid = $form_state->getValue('temp_file_id');
        if (!$fid) {
            $this->messenger()->addError($this->t('No valid file to import.'));
            return;
        }

        /** @var \Drupal\file\FileInterface $file */
        $file = File::load($fid);
        if (!$file) {
            $this->messenger()->addError($this->t('Could not load the file entity.'));
            return;
        }

        // Mark file permanent so Drupal doesn't garbage-collect it.
        $file->setPermanent();

        // Move it from temporary:// to public://.
        $destination_directory = 'public://taxonomy_import_export';
        $this->fileSystem->prepareDirectory(
            $destination_directory,
            FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS
        );
        $destination = $destination_directory . '/' . $file->getFilename();

        // Remove if a file with the same name exists (optional).
        $destination_realpath = $this->fileSystem->realpath($destination);
        if ($destination_realpath && file_exists($destination_realpath)) {
            $this->fileSystem->delete($destination);
        }

        try {
            $this->fileSystem->move($file->getFileUri(), $destination);
        }
        catch (FileException $e) {
            $this->messenger()->addError($this->t('Could not move uploaded file: @message', [
                '@message' => $e->getMessage(),
            ]));
            return;
        }

        // Update file entity to reflect the new URI.
        $file->setFileUri($destination);
        $file->save();

        // Parse the CSV and import taxonomy terms.
        $csv_string = file_get_contents($file->getFileUri());
        $vocabulary_id = $form_state->getValue('vocabulary');

        $vocab = \Drupal::entityTypeManager()
            ->getStorage('taxonomy_vocabulary')
            ->load($vocabulary_id);

        if (!$vocab) {
            $this->messenger()->addError($this->t('Invalid vocabulary selected.'));
            return;
        }

        $terms_data = $this->exportImportService->parseCsv($csv_string);
        $this->exportImportService->importTerms($vocab->id(), $terms_data);

        $this->messenger()->addStatus($this->t('Taxonomy terms have been successfully imported.'));
    }

}
